// Gui for GTK+ 2.0

#include <stdio.h>
#include <gtk/gtk.h>
#include <string.h>
#ifdef WIN32
#include <windows.h>
#endif
#include <stdlib.h>
#include <unistd.h>
#include "gtkGui.h"

/* Global variables are bad */
extern GdkColor     color_array[19];
extern GdkColormap *cmap;
GtkWidget          *file_selector_menu;

gboolean
configure_event (GtkWidget * graph, GdkEventConfigure * event, GdkPixmap * pixmap)
{
  /* we do nothing here */
  return TRUE;
}

/* Redraws pixmaps on expose event */
gboolean
expose_event (GtkWidget * graph, GdkEventExpose * event, GdkPixmap * pixmap)
{
  if (!pixmap)
    return TRUE;
  gdk_draw_pixmap (graph->window,
		   graph->style->white_gc,
		   pixmap,
		   event->area.x, event->area.y,
		   event->area.x, event->area.y,
		   event->area.width, event->area.height);
  return FALSE;
}

/*
    draws the currently selected file's graph 
    takes the average for 1/500th of the file
    and uses the range of bitrates to select 
    a color.
    Oggs are free-format bitrates, so colors
    are more accurate.
    mp3s are mostly a specific bitrate of 19
    choices, it's colors will be less accurate.
*/
void
draw_graph (statistic * mp3info2, Info * chosen1)
{
  float               divisor = (float) mp3info2->total_frames / 500;
  float               x = 0;
  int                 avg = 0, y = 0, zy = 0;
  GdkGC              *gc = NULL;
  GdkPixmap          *pixmap = NULL;

  gc = gdk_gc_new (chosen1->graph->window);
  /*  Mem leak, must bring
      drawable and table attach into function in order to destroy graph */

  chosen1->zz = 0;
  pixmap = gdk_pixmap_new (chosen1->graph->window, 500, 35, -1);
  gdk_gc_set_fill (gc, GDK_SOLID);

  /* increment for all frames */
  for (x = 0; x < mp3info2->total_frames; x += divisor) {
    /* Take average incrementing by our divisor (1/500th) */
    for (y = (int) x;
	 (y < (x + divisor)) && (y < mp3info2->total_frames); y++) {
      avg += place (mp3info2->bitrates[y]);
      zy++;
    }
    avg /= zy;
    zy = 0;
    gdk_gc_set_foreground (gc, &color_array[avg]);
    /* In case our for loop is (due to rounding)
       greater than 499, we dont want to overrun our pixmap length */
    if (chosen1->zz < 500) {
      gdk_draw_rectangle (pixmap, gc, TRUE, chosen1->zz, 0, 1, 35);
      gdk_draw_pixmap (chosen1->graph->window,
		       gc,
		       pixmap,
		       chosen1->zz, 0, chosen1->zz, 0, 1, 35);
    }
    avg = 0;
    chosen1->zz++;
  }
  chosen1->set = 1;
  chosen1->zz = 0;
  g_signal_connect (GTK_OBJECT (chosen1->graph), "configure_event",
		    G_CALLBACK (configure_event), pixmap);
  g_signal_connect (GTK_OBJECT (chosen1->graph), "expose_event",
		    G_CALLBACK (expose_event), pixmap);
  gtk_widget_queue_draw (chosen1->graph);
}


/*
	Shock, draws the pixmaps used in the key for both sides.
	Redrawn due to tables being destroyed to not leak memory.
*/
void
draw_keys (Info * chosen1)
{
  GtkWidget          *key = NULL;
  GdkPixmap          *pixmap = NULL;
  GdkGC              *gc2 = NULL;


  for (int x = 0; x < 19; x++) {
    key = gtk_drawing_area_new ();
    gtk_drawing_area_size (GTK_DRAWING_AREA (key), 10, 10);
    gtk_widget_show (key);
    gtk_table_attach (GTK_TABLE (chosen1->table2), key, 1, 2, x,
		      x + 1, GTK_SHRINK, GTK_SHRINK, 0, 0);
    gc2 = gdk_gc_new (key->window);
    pixmap = gdk_pixmap_new (key->window, 10, 10, -1);
    gdk_gc_set_foreground (gc2, &color_array[x]);
    gdk_gc_set_fill (gc2, GDK_SOLID);
    gdk_draw_rectangle (pixmap, gc2, TRUE, 0, 0, 10, 10);
    gdk_draw_pixmap (key->window, gc2, pixmap, 0, 0, 0, 0, 10, 10);
    gtk_widget_queue_draw (key);

    g_signal_connect (GTK_OBJECT (key), "configure_event",
		      G_CALLBACK (configure_event), pixmap);
    g_signal_connect (GTK_OBJECT (key), "expose_event",
		      G_CALLBACK (expose_event), pixmap);
  }
}

/*
	Depending on which file was selected, generate the labels for
	that side of the program.
*/
void
text_output (statistic * mp3info2, Info * chosen1)
{
  char                time[18];
  char               *name;
  char               *newname;
  char                sibit[40];
  char                amnt[10];
  char                kbit_array[19][8] =
    { "32kbit", "40kbit", "48kbit", "56kbit", "64kbit", "80kbit",
    "96kbit", "112kbit", "128kbit", "160kbit", "192kbit", "224kbit",
    "256kbit",
    "288kbit", "320kbit", "352kbit", "384kbit", "416kbit", "448kbit"
  };
  int                 x = 0, y = 0;
  GtkWidget          *label = NULL;

  /* Don't leak memory on subsequent queries */
  if (chosen1->table2) {
    gtk_widget_destroy (chosen1->table2);
    gtk_widget_destroy (chosen1->table3);
  }
  chosen1->table2 = gtk_table_new (3, 21, FALSE);
  gtk_table_set_col_spacing (GTK_TABLE (chosen1->table2), 0, 20);
  gtk_table_set_col_spacing (GTK_TABLE (chosen1->table2), 1, 20);
  gtk_container_add (GTK_CONTAINER (chosen1->fvbox), chosen1->table2);
  gtk_container_border_width (GTK_CONTAINER (chosen1->table2), 5);

  chosen1->table3 = gtk_table_new (2, 4, FALSE);
  gtk_table_set_col_spacing (GTK_TABLE (chosen1->table3), 0, 20);
  gtk_container_add (GTK_CONTAINER (chosen1->fvbox2), chosen1->table3);
  gtk_container_border_width (GTK_CONTAINER (chosen1->table3), 5);

  for (x = 0; x < (int) strlen (mp3info2->name); x++)
    if (mp3info2->name[x] == '/')
      y++;
  newname = new char[strlen (mp3info2->name) + y + 1];
  name = new char[strlen (mp3info2->name) + y + 8];
  y = 0;
  /* Because of gtk limitations with labels
     We have to make each filename a separate word */
  for (x = 0; x < (int) strlen (mp3info2->name); x++) {
    if (mp3info2->name[x] == '/') {
      newname[y] = '/';
      y++;
      newname[y] = ' ';
      y++;
    } else {
      newname[y] = mp3info2->name[x];
      y++;
    }
  }
  newname[y] = '\0';
  sprintf (name, "Name: %s", newname);

  label = gtk_label_new (name);
  delete[]name;
  delete[]newname;
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_FILL);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
  gtk_widget_set_usize (GTK_WIDGET (label), 500, -1);
  gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (chosen1->table3), label, 0,
		    2, 1, 2, GTK_FILL, GTK_SHRINK, 0, 0);
  if ((int) mp3info2->time_seconds % 60 <= 9)
    sprintf (time, "Time: %d:0%d", (int) mp3info2->time_seconds / 60,
	     (int) mp3info2->time_seconds % 60);
  else
    sprintf (time, "Time: %d:%d", (int) mp3info2->time_seconds / 60,
	     (int) mp3info2->time_seconds % 60);
  label = gtk_label_new (time);
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_FILL);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (chosen1->table3), label, 0,
		    1, 2, 3, GTK_FILL, GTK_SHRINK, 0, 0);
  sprintf (sibit, "Size: %.2fKB     ABR: %d", mp3info2->size,
	   (int) mp3info2->average_bitrate);
  label = gtk_label_new (sibit);
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_FILL);
  gtk_misc_set_alignment (GTK_MISC (label), 1, 0);
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (chosen1->table3), label, 1,
		    2, 2, 3, GTK_FILL, GTK_SHRINK, 0, 0);
  y = 3;
  /* generates the labels used in bitrate table */
  for (x = 0; x < 19; x++) {
    label = gtk_label_new (kbit_array[x]);
    gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
    gtk_widget_show (label);
    gtk_table_attach (GTK_TABLE (chosen1->table2), label,
		      0, 1, x, x + 1, GTK_FILL, GTK_SHRINK, 0, 0);
    y++;
    sprintf (amnt, "%d", mp3info2->amnt_bitrates[x]);
    label = gtk_label_new (amnt);
    gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_RIGHT);
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
    gtk_widget_show (label);
    gtk_table_attach (GTK_TABLE (chosen1->table2), label,
		      2, 3, x, x + 1, GTK_FILL, GTK_SHRINK, 0, 0);
    y++;
  }
  gtk_widget_show (chosen1->table2);
  gtk_widget_show (chosen1->table3);
}

/*
	Really weak and "MSish" way of checking filetypes for ogg and mp3s
	if true, calls functions to generate data for selected file
	otherwise, sends user back to file selection screen.
*/
void
is_mp3 (GtkWidget * ok_button, Info * chosen1)
{
  if (chosen1->mp3infoa != NULL) {
    delete[]chosen1->mp3infoa->bitrates;
    delete chosen1->mp3infoa;
    chosen1->mp3infoa = NULL;
  }
  chosen1->mp3infoa = new statistic;
  chosen1->mp3infoa->bitrates = NULL;
  chosen1->mp3infoa->name =
    (char *) gtk_file_selection_get_filename (GTK_FILE_SELECTION
					      (file_selector_menu));

#ifdef VORBIS
  if (chosen1->mp3infoa->name[(strlen (chosen1->mp3infoa->name) - 1)] == '3'
      || chosen1->mp3infoa->name[strlen (chosen1->mp3infoa->name) - 3] == 'o'
      || chosen1->mp3infoa->name[strlen (chosen1->mp3infoa->name) - 3] ==
      'O') {
#else
  if (chosen1->mp3infoa->name[(strlen (chosen1->mp3infoa->name) - 1)] == '3') {
#endif
    delete[]chosen1->selected_oldfile;
    chosen1->selected_oldfile =
      new char[strlen (chosen1->mp3infoa->name) + 1];
    strcpy (chosen1->selected_oldfile, chosen1->mp3infoa->name);
    get_file_data (chosen1->mp3infoa);
    text_output (chosen1->mp3infoa, chosen1);
    draw_keys (chosen1);
    draw_graph (chosen1->mp3infoa, chosen1);
  }
}

/*
	Generates a window for the file selection box, setting it to 
	the last directory where a file was analyzed 
*/
void
create_file_select (GtkWidget * button1, Info * chosen1)
{
  file_selector_menu =
    gtk_file_selection_new ("  Select an ogg/mp3 file to scan");
  if (chosen1->selected_oldfile)
    gtk_file_selection_set_filename (GTK_FILE_SELECTION (file_selector_menu),
				     chosen1->selected_oldfile);
  gtk_file_selection_hide_fileop_buttons (GTK_FILE_SELECTION
					  (file_selector_menu));
  gtk_widget_show (file_selector_menu);
  g_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (file_selector_menu)->
				ok_button), "clicked",
		    G_CALLBACK (is_mp3), chosen1);
  g_signal_connect_swapped (GTK_OBJECT
			    (GTK_FILE_SELECTION (file_selector_menu)->
			     cancel_button), "clicked",
			    G_CALLBACK (gtk_widget_destroy),
			    file_selector_menu);
  g_signal_connect_swapped (GTK_OBJECT
			    (GTK_FILE_SELECTION (file_selector_menu)->
			     ok_button), "clicked",
			    G_CALLBACK (gtk_widget_destroy),
			    file_selector_menu);

}

/* closes the application */
static void
close_application (GtkWidget * widget, gpointer data)
{
  gtk_main_quit ();
}

/* Does all the gtk stuff */
void
gtk_interface (int argc, char *argv[])
{
  Info                chosen, chosen2;	/* structures for the two files */

  GtkWidget          *window = NULL;	/* Main window */
  GtkWidget          *button = NULL;	/* buttons */
  GtkWidget          *table = NULL;	/* table to organize file1 and file2 */
  GtkWidget          *graphbox = NULL;  /* Holds pixmaps */
  GtkWidget          *graph2box = NULL; /* Holds pixmaps */

  gtk_init (&argc, &argv);

  cmap = gdk_colormap_get_system ();
  set_graph_colors ();
  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  g_signal_connect (GTK_OBJECT (window), "destroy",
		    G_CALLBACK (close_application), NULL);

  gtk_window_set_title (GTK_WINDOW (window), "Mp3stat");
  gtk_container_border_width (GTK_CONTAINER (window), 5);
  /* Main table for gui 2x7 */
  table = gtk_table_new (2, 8, FALSE);
  gtk_container_add (GTK_CONTAINER (window), table);
  chosen.fvbox = gtk_frame_new ("File 1 bitrate breakdown");
  gtk_table_attach (GTK_TABLE (table), chosen.fvbox, 0, 1, 5, 6,
		    GTK_SHRINK, GTK_SHRINK, 0, 0);
  chosen2.fvbox = gtk_frame_new ("File 2 bitrate breakdown");
  gtk_table_attach (GTK_TABLE (table), chosen2.fvbox, 1, 2, 5, 6,
		    GTK_SHRINK, GTK_SHRINK, 0, 0);
  chosen.fvbox2 = gtk_frame_new ("File 1 Summary");
  gtk_table_attach (GTK_TABLE (table), chosen.fvbox2, 0, 2, 4, 5,
		    GTK_FILL, GTK_SHRINK, 0, 0);
  chosen2.fvbox2 = gtk_frame_new ("File 2 Summary");
  gtk_table_attach (GTK_TABLE (table), chosen2.fvbox2, 0, 2, 6, 7,
		    GTK_FILL, GTK_SHRINK, 0, 0);
  gtk_widget_show (chosen.fvbox2);
  gtk_widget_show (chosen.fvbox);
  gtk_widget_show (chosen2.fvbox2);
  gtk_widget_show (chosen2.fvbox);
  button = gtk_button_new_with_label ("Scan file #1");
  g_signal_connect (GTK_OBJECT (button), "clicked",
		    G_CALLBACK (create_file_select), &chosen);
  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
  gtk_widget_show (button);
  gtk_table_attach (GTK_TABLE (table), button, 1, 2, 0, 1,
		    GTK_SHRINK, GTK_SHRINK, 0, 0);
  gtk_widget_grab_default (button);
  chosen.graph = gtk_drawing_area_new ();
  gtk_drawing_area_size (GTK_DRAWING_AREA (chosen.graph), 500, 35);
  gtk_widget_show (chosen.graph);
  graphbox = gtk_event_box_new ();
  gtk_event_box_set_visible_window (GTK_EVENT_BOX (graphbox), FALSE);
  gtk_event_box_set_above_child (GTK_EVENT_BOX (graphbox), TRUE);
  gtk_container_add (GTK_CONTAINER (graphbox), chosen.graph);
  g_signal_connect (GTK_OBJECT (graphbox), "button_press_event",
		    G_CALLBACK (detailed_info), &chosen);
  gtk_widget_show (graphbox);
  chosen2.graph = gtk_drawing_area_new ();
  gtk_drawing_area_size (GTK_DRAWING_AREA (chosen2.graph), 500, 35);
  gtk_widget_show (chosen2.graph);
  graph2box = gtk_event_box_new ();
  gtk_event_box_set_visible_window (GTK_EVENT_BOX (graph2box), FALSE);
  gtk_event_box_set_above_child (GTK_EVENT_BOX (graph2box), TRUE);
  gtk_container_add (GTK_CONTAINER (graph2box), chosen2.graph);
  g_signal_connect (GTK_OBJECT (graph2box), "button_press_event",
		    G_CALLBACK (detailed_info), &chosen2);
  gtk_widget_show (graph2box);
  button = gtk_button_new_with_label ("Scan file #2");
  g_signal_connect (GTK_OBJECT (button), "clicked",
		    G_CALLBACK (create_file_select), &chosen2);
  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
  gtk_widget_show (button);
  gtk_widget_show (table);
  gtk_widget_show (window);
  gtk_table_attach (GTK_TABLE (table), button, 1, 2, 3, 4,
		    GTK_SHRINK, GTK_SHRINK, 0, 0);
  gtk_widget_grab_default (button);
  gtk_table_attach (GTK_TABLE (table), graphbox, 0, 2, 1, 2,
		    GTK_SHRINK, GTK_SHRINK, 0, 0);
  gtk_table_attach (GTK_TABLE (table), graph2box, 0, 2, 2, 3,
		    GTK_SHRINK, GTK_SHRINK, 0, 0);

  gtk_main ();
}

/* calls for gtk output */
int
main (int argc, char *argv[])
{
#ifndef WIN32
  int                 index = 0;
  struct statistic    mp3info;
  mp3info.bitrates = NULL;
  if (argc >= 2) {
    if (strcmp (argv[1], "-h") == 0 || strcmp (argv[1], "-H") == 0
	|| strcmp (argv[1], "--help") == 0) {
      printf ("mp3stat   version %s \n", VERSION);
      printf ("Usage:\nmp3stat (options) \n	-h/-H/--help");
      printf ("	            This help screen\n	-v/-V/-");
      printf ("-version	            Display version\n    ");
      printf ("    -b/-B/--batch 1/0 1/0 1/0   See README\n");
      exit (0);
    }
    if (strcmp (argv[1], "-v") == 0 || strcmp (argv[1], "-V") == 0
	|| strcmp (argv[1], "--version") == 0) {
      printf ("mp3stat %s\n", VERSION);
      exit (0);
    }
    if (strcmp (argv[1], "-b") == 0 || strcmp (argv[1], "-B") == 0
	|| strcmp (argv[1], "--batch") == 0) {
      for (index = 2; index <= argc - 1; index++) {
	mp3info.name = argv[index];
	if (get_file_data (&mp3info) == 0) {
	  if (strcmp (argv[2], "1") == 0)
	    printf ("%s  ", mp3info.name);
	  if (strcmp (argv[3], "1") == 0)
	    printf ("%.2f  ", mp3info.time_seconds);
	  if (strcmp (argv[4], "1") == 0)
	    printf ("%i  ", (int) mp3info.average_bitrate);
	  if (strcmp (argv[2], "0") != 0 && strcmp (argv[2], "1") != 0)
	    printf ("%s   %.2f  %i  ", mp3info.name,
		    mp3info.time_seconds, (int) mp3info.average_bitrate);
	  printf ("\n");
	  delete[]mp3info.bitrates;
	}
      }
      return 0;
    }
  } else {
#endif
#ifdef WIN32
    SetEnvironmentVariable ("HOME", ".");
    putenv ("HOME=.");
#endif
    gtk_interface (argc, argv);
    return 0;
#ifndef WIN32
  }
#endif
}
